==> coap-node-0.1.8.tgz: package/lib/components/cutils.js <==
'use strict';

var urlParser = require('url').parse,
    lwm2mId = require('lwm2m-id'),
    _ = require('busyman');

var cutils = {};

cutils.getTime = function () {
    return Math.round(new Date().getTime()/1000);
};

/*********************************************************
 * lwm2m-id utils                                        *
 *********************************************************/
cutils.oidKey = function (oid) {
    var oidItem = lwm2mId.getOid(oid);
    return oidItem ? oidItem.key : oid;
};

cutils.oidNumber = function (oid) {
    var oidItem = lwm2mId.getOid(oid);

    oidItem = oidItem ? oidItem.value : parseInt(oid);   

    if (_.isNaN(oidItem))
        oidItem = oid;

    return oidItem;
};

cutils.ridKey = function (oid, rid) {
    var ridItem = lwm2mId.getRid(oid, rid);

    if (_.isUndefined(rid))
        rid = oid;

    return ridItem ? ridItem.key : rid;
};

cutils.ridNumber = function (oid, rid) {
    var ridItem = lwm2mId.getRid(oid, rid);

    if (_.isUndefined(rid))
        rid = oid;

    ridItem = ridItem ? ridItem.value : parseInt(rid);   

    if (_.isNaN(ridItem))
        ridItem = rid;

    return ridItem;
};

/*********************************************************
 * path utils                                            *
 *********************************************************/
cutils.buildRptAttr = function (req) {      // 'pmin=10&pmax=60'
    var allowedAttrs = [ 'pmin', 'pmax', 'gt', 'lt', 'stp' ],
        attrs = {},
        query = urlParser(req.url).query,
        queryParams = query.split('&');

    _.forEach(queryParams, function (queryParam, idx) {
        queryParams[idx] = queryParam.split('=');       // [[ pmin, 10 ], [ pmax, 60 ]]
    });

    _.forEach(queryParams, function(queryParam) {     
        if (_.includes(allowedAttrs, queryParam[0])) {
            attrs[queryParam[0]] = Number(queryParam[1]);
        } else {
            return false;
        }
    });

    return attrs;   // { pmin: 10, pmax:60 }
};

cutils.buildUpdateQuery = function (attrs) {    // { lifetime: 81000, version: 'v0.1.2' }
    var query = '';

    _.forEach(attrs, function (val, key) {
        if (key === 'lifetime' || key === 'lt')
            query += 'lt=' + val + '&';
        else if (key === 'version' || key === 'lwm2m')
            query += 'lwm2m=' + val + '&';
    });

    if (query[query.length-1] === '&')           
        query = query.slice(0, query.length-1);

    return query;   // 'lt=81000&lwm2m=v0.1.2'
};

cutils.getArrayArgus = function (argusInPlain) {    // 10,15,'xx'
    var argusInArray = [],
        notallowed = [' ', '"', "'", '\\'],
        isAnyNotallowed = false;

    function chkCharSyntax(string) {
        _.forEach(notallowed, function (val) {
            if (_.includes(string, val))
                isAnyNotallowed = true;
        });
    }

    if (argusInPlain.length === 0)
        return [];

    if (Number(argusInPlain))
        argusInPlain = argusInPlain.toString();

    _.forEach(argusInPlain.split(','), function (argu) {
        if (Number(argu)) {
            argusInArray.push(Number(argu));
        } else if (_.includes(argu, '=')) {
            argusInArray.push(argu.split('=')[1].slice(1, argu.length - 1));
            chkCharSyntax(argusInArray[argusInArray.length - 1]);
        } else {
            argusInArray.push(argu.slice(1, argu.length - 1));
            chkCharSyntax(argusInArray[argusInArray.length - 1]);
        }
    });

    if (isAnyNotallowed)
        return false;
    else
        return argusInArray;    // [10, 15, 'xx']
};

/*********************************************************
 * path utils                                            *
 *********************************************************/
cutils.chkPathSlash = function (path) {
    if (path.charAt(0) === '/') {
        return path;
    } else {
        return '/' + path;
    }
};

cutils.urlParser = function (url) {
    var urlObj = {
        pathname: url.split('?')[0],
        query: url.split('?')[1]
    };

    return urlObj;
};

cutils.pathSlashParser = function (url) {
    var path = urlParser(url).pathname,
        pathArray = path.split('/');       // '/x/y/z'

    if (pathArray[0] === '') 
        pathArray = pathArray.slice(1);

    if (pathArray[pathArray.length-1] === '')           
        pathArray = pathArray.slice(0, pathArray.length-1);

    return pathArray;  // ['x', 'y', 'z']
};

cutils.getSoKeyObj = function (url) {
    var pathArray = this.pathSlashParser(url),   // '/1/2/3'
        pathObj = {},
        oid,
        rid;

    if (url) {
        if (pathArray[0]) {    //oid
            oid = this.oidKey(pathArray[0]);
            pathObj.oid = oid;

            if (pathArray[1]) {    //iid
                pathObj.iid = + pathArray[1]; 

                if (pathArray[2]) {    //rid
                    rid = this.ridKey(oid, pathArray[2]);
                    pathObj.rid = rid;
                } 
            }
        }
    }

    return pathObj;     // {oid:'lwm2mServer', iid: '2', rid: 'defaultMaxPeriod'}
};

cutils.pathDateType = function (path) {
    var pathArray = this.pathSlashParser(path),
        dateType = [ 'so', 'object', 'instance', 'resource' ][pathArray.length];
    return dateType;
};

/*********************************************************
 * JSON utils                                            *
 *********************************************************/
cutils.encodeJsonObj = function (basePath, value) {
    var self = this,
        objInJson = { 
            bn: this.chkPathSlash(basePath), 
            'e': [] 
        },
        pathType = this.pathDateType(basePath),
        pathArray = this.pathSlashParser(basePath),
        oid = pathArray[0];

    if (pathType === 'object') {
        if (!_.isPlainObject(value)) throw new TypeError('value should be a object.');
        _.forEach(value, function (iObj, iid) {
            _.forEach(iObj, function (resrc, rid) {
                if (_.isPlainObject(resrc)) {
                    _.forEach(resrc, function (r, riid) {
                        var data = self.encodeJsonValue(iid + '/' + self.ridNumber(oid, rid) + '/' + riid, r);
                        objInJson.e.push(data);
                    });
                } else {
                    var data = self.encodeJsonValue(iid + '/' + self.ridNumber(oid, rid), resrc);
                    objInJson.e.push(data);
                }
            });
        });
    } else if (pathType === 'instance') {
        if (!_.isPlainObject(value)) throw new TypeError('value should be a object.');
        _.forEach(value, function (resrc, rid) {
            if (_.isPlainObject(resrc)) {
                _.forEach(resrc, function (r, riid) {
                    var data = self.encodeJsonValue(self.ridNumber(oid, rid) + '/' + riid, r);
                    objInJson.e.push(data);
                });
            } else {
                var data = self.encodeJsonValue(self.ridNumber(oid, rid), resrc);
                objInJson.e.push(data);
            }
        });
    } else if (pathType === 'resource') {
         if (_.isPlainObject(value)) {
            _.forEach(value, function (r, riid) {
                var data = self.encodeJsonValue(riid, r);
                objInJson.e.push(data);
            });
        } else {
            if (value instanceof Date) value = Number(value);
            var data = self.encodeJsonValue('', value);
            objInJson.e.push(data);
        }
    }

    return objInJson;
};

cutils.encodeJsonValue = function (basePath, value) {
    var val = {};

    if (_.isNil(value)) {
        value = basePath;
        basePath = undefined;
    } else {
        val.n = basePath.toString();
    }

    if (_.isNumber(value)) {
        val.v = Number(value);
    } else if (_.isString(value)) {
        val.sv = String(value);
    } else if (value instanceof Date) {
        val.v = Number(value);       
    } else if (_.isBoolean(value)) {
        val.bv = Boolean(value);
    } else if (_.isPlainObject(value)) {
        val.ov = value;     // [TODO] objlnk
    }

    return val;
};

cutils.decodeJsonObj = function (basePath, value) {
    var self = this,
        data = {},
        pathType = this.pathDateType(basePath),
        oid = this.getSoKeyObj(basePath).oid,
        rid;

    if (value.e) {
        switch (pathType) {
            case 'object':         // obj
                _.forEach(value.e, function (resrc) {
                    var path = resrc.n.split('/'),          // [iid, rid[, riid]]
                        val;

                    if (!_.isUndefined(resrc.v)) {
                        val = resrc.v;
                    } else if (!_.isUndefined(resrc.sv)) {
                        val = resrc.sv;
                    } else if (!_.isUndefined(resrc.bv)) {
                        val = resrc.bv;
                    } else if (!_.isUndefined(resrc.ov)) {
                        val = resrc.ov;     // [TODO] objlnk
                    }

                    if (path[0] === '')
                        path = path.slice(1);

                    if (path[path.length - 1] === '')
                        path = path.slice(0, path.length - 1);

                    if (path[0] && !_.has(data, path[0]))
                        data[path[0]] = {};

                    rid = self.ridKey(oid, path[1]);

                    if (rid && !_.has(data, [path[0], rid])) {
                        if (path[2]) {
                            data[path[0]][rid] = {};
                            data[path[0]][rid][path[2]] = val;
                        } else {
                            data[path[0]][rid] = val;
                        }
                    }
                });
                break;
            case 'instance':         // inst
                _.forEach(value.e, function (resrc) {
                    var path = resrc.n.split('/'),          // [rid[, riid]]
                        val;

                    if (!_.isUndefined(resrc.v)) {
                        val = resrc.v;
                    } else if (!_.isUndefined(resrc.sv)) {
                        val = resrc.sv;
                    } else if (!_.isUndefined(resrc.bv)) {
                        val = resrc.bv;
                    } else if (!_.isUndefined(resrc.ov)) {
                        val = resrc.ov;     // [TODO] objlnk
                    }

                    if (path[0] === '')
                        path = path.slice(1);

                    if (path[path.length - 1] === '')
                        path = path.slice(0, path.length - 1);

                    rid = self.ridKey(oid, path[0]);

                    if (rid && !_.has(data, rid)) {
                        if (path[1]) {
                            data[rid] = {};
                            data[rid][path[1]] = val;
                        } else {
                            data[rid] = val;
                        }
                    }
                });
                break;
            case 'resource':         // resrc
                _.forEach(value.e, function (resrc) {
                    var path = resrc.n,          // [[riid]]
                        val;

                    if (!_.isUndefined(resrc.v)) {
                        val = resrc.v;
                    } else if (!_.isUndefined(resrc.sv)) {
                        val = resrc.sv;
                    } else if (!_.isUndefined(resrc.bv)) {
                        val = resrc.bv;
                    } else if (!_.isUndefined(resrc.ov)) {
                        val = resrc.ov;     // [TODO] objlnk
                    }

                    if (path === '') {
                        data = val;
                        return ;
                    } else if (path && !_.has(data, path)) {
                        data[path] = val;
                    }
                });
                break;
        default:
            break;
        }
    } else if (!_.isPlainObject(value) && pathType !== 'resource') {
        if (!_.isPlainObject(value)) throw new TypeError('value should be a object.');
    } else if (!_.isPlainObject(value) && pathType === 'resource') {
        data = value;
    } else {    // value is object and but not LWM2M format
        data = value;
    }
    
    return data;
};

module.exports = cutils;

==> coap-node-0.2.1.tgz: package/lib/components/cutils.js <==
'use strict';

var urlParser = require('url').parse,
    lwm2mId = require('lwm2m-id'),
    lwm2mCodec = require('lwm2m-codec'),
    _ = require('busyman');

var cutils = {};

cutils.getTime = function () {
    return Math.round(new Date().getTime()/1000);
};

/*********************************************************
 * lwm2m-id utils                                        *
 *********************************************************/
cutils.oidKey = function (oid) {
    var oidItem = lwm2mId.getOid(oid);
    return oidItem ? oidItem.key : oid;
};

cutils.oidNumber = function (oid) {
    var oidItem = lwm2mId.getOid(oid);

    oidItem = oidItem ? oidItem.value : parseInt(oid);

    if (_.isNaN(oidItem))
        oidItem = oid;

    return oidItem;
};

cutils.ridKey = function (oid, rid) {
    var ridItem = lwm2mId.getRid(oid, rid);

    if (_.isUndefined(rid))
        rid = oid;

    return ridItem ? ridItem.key : rid;
};

cutils.ridNumber = function (oid, rid) {
    var ridItem = lwm2mId.getRid(oid, rid);

    if (_.isUndefined(rid))
        rid = oid;

    ridItem = ridItem ? ridItem.value : parseInt(rid);

    if (_.isNaN(ridItem))
        ridItem = rid;

    return ridItem;
};

/*********************************************************
 * path utils                                            *
 *********************************************************/
cutils.buildRptAttr = function (req) {      // 'pmin=10&pmax=60'
    var allowedAttrs = [ 'pmin', 'pmax', 'gt', 'lt', 'stp' ],
        attrs = {},
        query = urlParser(req.url).query,
        queryParams = query.split('&');

    _.forEach(queryParams, function (queryParam, idx) {
        queryParams[idx] = queryParam.split('=');       // [[ pmin, 10 ], [ pmax, 60 ]]
    });

    _.forEach(queryParams, function(queryParam) {
        if (_.includes(allowedAttrs, queryParam[0])) {
            attrs[queryParam[0]] = Number(queryParam[1]);
        } else {
            return false;
        }
    });

    return attrs;   // { pmin: 10, pmax:60 }
};

cutils.buildUpdateQuery = function (attrs) {    // { lifetime: 81000, version: 'v0.1.2' }
    var query = '';

    _.forEach(attrs, function (val, key) {
        if (key === 'lifetime' || key === 'lt')
            query += 'lt=' + val + '&';
        else if (key === 'version' || key === 'lwm2m')
            query += 'lwm2m=' + val + '&';
    });

    if (query[query.length-1] === '&')
        query = query.slice(0, query.length-1);

    return query;   // 'lt=81000&lwm2m=v0.1.2'
};

cutils.getArrayArgus = function (argusInPlain) {    // 10,15,'xx'
    var argusInArray = [],
        notallowed = [' ', '"', "'", '\\'],
        isAnyNotallowed = false;

    function chkCharSyntax(string) {
        _.forEach(notallowed, function (val) {
            if (_.includes(string, val))
                isAnyNotallowed = true;
        });
    }

    if (argusInPlain.length === 0)
        return [];

    if (Number(argusInPlain))
        argusInPlain = argusInPlain.toString();

    _.forEach(argusInPlain.split(','), function (argu) {
        if (Number(argu)) {
            argusInArray.push(Number(argu));
        } else if (_.includes(argu, '=')) {
            argusInArray.push(argu.split('=')[1].slice(1, argu.length - 1));
            chkCharSyntax(argusInArray[argusInArray.length - 1]);
        } else {
            argusInArray.push(argu.slice(1, argu.length - 1));
            chkCharSyntax(argusInArray[argusInArray.length - 1]);
        }
    });

    if (isAnyNotallowed)
        return false;
    else
        return argusInArray;    // [10, 15, 'xx']
};

/*********************************************************
 * path utils                                            *
 *********************************************************/
cutils.chkPathSlash = function (path) {
    if (path.charAt(0) === '/') {
        return path;
    } else {
        return '/' + path;
    }
};

cutils.urlParser = function (url) {
    var urlObj = {
        pathname: url.split('?')[0],
        query: url.split('?')[1]
    };

    return urlObj;
};

cutils.getPathArray = function (url) {
    var path = urlParser(url).pathname,
        pathArray = path.split('/');       // '/x/y/z'

    if (pathArray[0] === '')
        pathArray = pathArray.slice(1);

    if (pathArray[pathArray.length-1] === '')
        pathArray = pathArray.slice(0, pathArray.length-1);

    return pathArray;  // ['x', 'y', 'z']
};

cutils.getPathIdKey = function (url) {
    var pathArray = this.getPathArray(url),   // '/1/2/3'
        pathObj = {},
        oid,
        rid;

    if (url) {
        if (pathArray[0]) {    //oid
            oid = this.oidKey(pathArray[0]);
            pathObj.oid = oid;

            if (pathArray[1]) {    //iid
                pathObj.iid = + pathArray[1];

                if (pathArray[2]) {    //rid
                    rid = this.ridKey(oid, pathArray[2]);
                    pathObj.rid = rid;
                }
            }
        }
    }

    return pathObj;     // {oid:'lwm2mServer', iid: '2', rid: 'defaultMaxPeriod'}
};

cutils.getPathDateType = function (path) {
    var pathArray = this.getPathArray(path),
        dateType = [ 'so', 'object', 'instance', 'resource' ][pathArray.length];
    return dateType;
};

/*********************************************************
 * Link utils                                            *
 *********************************************************/
cutils.encodeLinkFormat = function (path, value, attrs) {
    return lwm2mCodec.encode('link', path, value, attrs);
};
/*********************************************************
 * TLV utils                                             *
 *********************************************************/
cutils.encodeTlv = function (path, value) {
    return lwm2mCodec.encode('tlv', path, value);
};

cutils.decodeTlv = function (path, value) {
    return lwm2mCodec.decode('tlv', path, value);
};

/*********************************************************
 * JSON utils                                            *
 *********************************************************/
cutils.encodeJson = function (path, value) {
    return lwm2mCodec.encode('json', path, value);
};

cutils.decodeJson = function (path, value) {
    return lwm2mCodec.decode('json', path, value);
};

module.exports = cutils;
